"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const constants_1 = require("./constants");
const getComment_1 = require("./getComment");
const getEnum_1 = require("./getEnum");
const getEnumFromDescription_1 = require("./getEnumFromDescription");
const getModelProperties_1 = require("./getModelProperties");
const getType_1 = require("./getType");
function getModel(openApi, definition, isProperty = false, name = '') {
    const model = {
        name: name,
        export: 'interface',
        type: constants_1.PrimaryType.OBJECT,
        base: constants_1.PrimaryType.OBJECT,
        template: null,
        link: null,
        description: getComment_1.getComment(definition.description),
        isProperty: isProperty,
        isReadOnly: definition.readOnly === true,
        isNullable: definition.nullable === true,
        isRequired: false,
        imports: [],
        extends: [],
        enum: [],
        enums: [],
        properties: [],
    };
    if (definition.$ref) {
        const definitionRef = getType_1.getType(definition.$ref);
        model.export = 'reference';
        model.type = definitionRef.type;
        model.base = definitionRef.base;
        model.template = definitionRef.template;
        model.imports.push(...definitionRef.imports);
        return model;
    }
    if (definition.enum) {
        const enumerators = getEnum_1.getEnum(definition.enum);
        if (enumerators.length) {
            model.export = 'enum';
            model.type = constants_1.PrimaryType.STRING;
            model.base = constants_1.PrimaryType.STRING;
            model.enum.push(...enumerators);
            return model;
        }
    }
    if ((definition.type === 'int' || definition.type === 'integer') && definition.description) {
        const enumerators = getEnumFromDescription_1.getEnumFromDescription(definition.description);
        if (enumerators.length) {
            model.export = 'enum';
            model.type = constants_1.PrimaryType.NUMBER;
            model.base = constants_1.PrimaryType.NUMBER;
            model.enum.push(...enumerators);
            return model;
        }
    }
    if (definition.type === 'array' && definition.items) {
        if (definition.items.$ref) {
            const arrayItems = getType_1.getType(definition.items.$ref);
            model.export = 'array';
            model.type = arrayItems.type;
            model.base = arrayItems.base;
            model.template = arrayItems.template;
            model.imports.push(...arrayItems.imports);
            return model;
        }
        else {
            const arrayItems = getModel(openApi, definition.items, true);
            model.export = 'array';
            model.type = arrayItems.type;
            model.base = arrayItems.base;
            model.template = arrayItems.template;
            model.link = arrayItems;
            model.imports.push(...arrayItems.imports);
            return model;
        }
    }
    if (definition.type === 'object' && definition.additionalProperties && typeof definition.additionalProperties === 'object') {
        if (definition.additionalProperties.$ref) {
            const additionalProperties = getType_1.getType(definition.additionalProperties.$ref);
            model.export = 'dictionary';
            model.type = additionalProperties.type;
            model.base = additionalProperties.base;
            model.template = additionalProperties.template;
            model.imports.push(...additionalProperties.imports);
            model.imports.push('Dictionary');
            return model;
        }
        else {
            const additionalProperties = getModel(openApi, definition.additionalProperties);
            model.export = 'dictionary';
            model.type = additionalProperties.type;
            model.base = additionalProperties.base;
            model.template = additionalProperties.template;
            model.link = additionalProperties;
            model.imports.push(...additionalProperties.imports);
            model.imports.push('Dictionary');
            return model;
        }
    }
    if (definition.type === 'object') {
        model.export = 'interface';
        model.type = constants_1.PrimaryType.OBJECT;
        model.base = constants_1.PrimaryType.OBJECT;
        if (definition.allOf) {
            definition.allOf.forEach(parent => {
                if (parent.$ref) {
                    const parentRef = getType_1.getType(parent.$ref);
                    model.extends.push(parentRef.type);
                    model.imports.push(parentRef.base);
                }
                if (parent.type === 'object' && parent.properties) {
                    const properties = getModelProperties_1.getModelProperties(openApi, parent);
                    properties.forEach(property => {
                        model.properties.push(property);
                        model.imports.push(...property.imports);
                        if (property.export === 'enum') {
                            model.enums.push(property);
                        }
                    });
                }
            });
        }
        if (definition.properties) {
            const properties = getModelProperties_1.getModelProperties(openApi, definition);
            properties.forEach(property => {
                model.properties.push(property);
                model.imports.push(...property.imports);
                if (property.export === 'enum') {
                    model.enums.push(property);
                }
            });
        }
        return model;
    }
    // If the schema has a type than it can be a basic or generic type.
    if (definition.type) {
        const definitionType = getType_1.getType(definition.type);
        model.export = 'generic';
        model.type = definitionType.type;
        model.base = definitionType.base;
        model.template = definitionType.template;
        model.imports.push(...definitionType.imports);
        return model;
    }
    return model;
}
exports.getModel = getModel;
